local Menu = Game:addState("Menu")

function Menu:enteredState()
    g.setDefaultFilter('nearest', 'nearest', 0)
    background = self.preloaded_images['printer_normal_no_paper.png']
    title = self.preloaded_images['title.png']

    btn_sound = self.preloaded_sounds['btn_hover.wav']

    playbtn = self.preloaded_images['playbtn.png']
    playbtn_hover = false
    exitbtn = self.preloaded_images['exitbtn.png']
    exitbtn_hover = false
    playbtn_offset = math.random()
    exitbtn_offset = math.random()
    menu_time = 0
    transition_time = 0
    transition_happening = false
end

function Menu:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(background, 0, 0, 0, 1, 1)
    g.draw(title, 0, 0, 0, 1+(math.sin(menu_time)*0.15), 1+(math.sin(menu_time)*0.15))

    g.setFont(self.preloaded_fonts["04b03_20"])
    g.setColor(150,150,150,255)
    g.print("Printer Battle Music by: Pant0don", 10, screen_h-90)
    g.setColor(91, 110, 225, 255)
    g.print("Download at: http://opengameart.org/content/glitchy-battle", 10, screen_h-70)
    g.setColor(255, 255, 255, 255)

    g.setColor(150,150,150,255)
    g.print("Engine sound and power down sound by: qubodup", 10, screen_h-40)
    g.setColor(91, 110, 225, 255)
    g.print("Download at: http://opengameart.org/users/qubodup", 10, screen_h-20)
    g.setColor(255, 255, 255, 255)

    g.draw(playbtn, (screen_w/2), (screen_h-250), (math.sin(menu_time-playbtn_offset)*0.10), (playbtn_hover and 1.5 or 1), (playbtn_hover and 1.5 or 1), (playbtn:getWidth()/2), (playbtn:getHeight()/2))
    g.draw(exitbtn, (screen_w/2), (screen_h-150), (math.sin(menu_time-exitbtn_offset)*0.10), (exitbtn_hover and 1.5 or 1), (exitbtn_hover and 1.5 or 1), (exitbtn:getWidth()/2), (exitbtn:getHeight()/2))


    local alpha = cap(255*menu_time, 0,255)
    g.setColor(0, 0, 0, 255-alpha)
    g.rectangle("fill", 0, 0, screen_w, screen_h)

    if transition_happening then
        local alpha = cap(255*transition_time, 0,255)
        g.setColor(0, 0, 0, alpha)
        g.rectangle("fill", 0, 0, screen_w, screen_h)
    end
end

function Menu:update(dt)
    menu_time = menu_time + dt
    if transition_happening then
        transition_time = transition_time + dt
        if transition_time > 1.3 then
            self:gotoState('Intro')
        end
    end
end

function Menu:mousepressed(x, y, button, isTouch)
    if button > 1 then
        return
    end

    if playbtn_hover then
        transition_happening = true
    end

    if exitbtn_hover then
        love.event.push("quit")
    end
end

function Menu:mousemoved(x, y, dx, dy)
    local play_startx = ((screen_w/2)-((playbtn_hover and (playbtn:getWidth()*1.5) or playbtn:getWidth())/2))
    local play_starty = ((screen_h-250)-((playbtn_hover and (playbtn:getHeight()*1.5) or playbtn:getHeight())/2))

    if x >= play_startx and x <= play_startx + (playbtn_hover and (playbtn:getWidth()*1.5) or playbtn:getWidth()) then
        if y >= play_starty and y <= play_starty + (playbtn_hover and (playbtn:getHeight()*1.5) or playbtn:getHeight()) then
            if playbtn_hover == false then
                btn_sound:play()
            end
            playbtn_hover = true
        else
            playbtn_hover = false
        end
    else
        playbtn_hover = false
    end

    local exit_startx = ((screen_w/2)-((exitbtn_hover and (exitbtn:getWidth()*1.5) or exitbtn:getWidth())/2))
    local exit_starty = ((screen_h-150)-((exitbtn_hover and (exitbtn:getHeight()*1.5) or exitbtn:getHeight())/2))

    if x >= exit_startx and x <= exit_startx + (exitbtn_hover and (exitbtn:getWidth()*1.5) or exitbtn:getWidth()) then
        if y >= exit_starty and y <= exit_starty + (exitbtn_hover and (exitbtn:getHeight()*1.5) or exitbtn:getHeight()) then
            if exitbtn_hover == false then
                btn_sound:play()
            end
            exitbtn_hover = true
        else
            exitbtn_hover = false
        end
    else
        exitbtn_hover = false
    end
end

function Menu:exitedState()

end
