local Exorcism = Printer:addState("Exorcism")

function Exorcism:enteredState()
    animation_time = 0
    paper = nil


    penta_random = math.random()
    prev_penta_random = penta_random
    penta_count = 0
    next_penta_time = 4

    printer_hp = 1.0
    player_hp = 1.0
    stars = {}

    self.lcd:gotoState("Empty")
    self.battle_music:setLooping(true)
    self.battle_music:play()
    self.battle_music:setVolume(0.5)

    self.game:setCountdownEnabled(false)

    min_movement = 5
    max_movement = 10

    printer_dead_time = 0
    player_dead_time = 0

    self.exorcism_happend = true
end

function Exorcism:hasPaper()
    return true
end

function Exorcism:hasInkt()
    return true
end

function Exorcism:isHatchOpen()
    return false
end

function Exorcism:mousereleased(x, y, button, isTouch)
    if button == 1 then
        for i,v in ipairs(stars) do
            if v.pressed and not v.disabled and not v.hit then
                v.hit = true
                v.pressed = false
                v.hover = false
            end
        end
    end
end

function Exorcism:mousepressed(x, y, button, isTouch)
    if button == 1 then
        for i,v in ipairs(stars) do
            if v.hover and not v.disabled then
                v.pressed = true
            end
        end
    end
end


function Exorcism:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(self.bg, 0, 0, 0, 1, 1)

    local alpha = cap(255*(animation_time*0.30), 0,255)
    local pentagram = cap(255*-(animation_time*0.30), -255, 0)
    local yd = cap(255*(animation_time*0.10), 0, 150)
    local fill = 100

    local hp_bar = ((screen_w-68) * printer_hp) * cap((animation_time*0.20), 0, 1)
    local player_hp_bar = ((screen_w-68) * player_hp) * cap((animation_time*0.20), 0, 1)

    if printer_hp == 0 then
        time_since_dead = animation_time - printer_dead_time

        alpha = alpha + cap(255*-(time_since_dead*0.30), -255, 0)
        pentagram = pentagram + cap(255*(time_since_dead*0.30), 0, 255)
        yd = yd + cap(255*-(time_since_dead*0.10), -150, 0)
    end


    g.setColor(0, 0, 0, alpha)
    g.rectangle('fill', 0, 0, screen_w, screen_h)
    if (animation_time*0.30) > 2 then
        g.setColor(255, 255, 255, 0-pentagram)
        g.draw(self.pentagram, 0, 0, 0, 1, 1)
    end
    g.setColor(255, 255, 255, 255)


    xr = math.random(min_movement, max_movement)

    self.lcd:setPrinterCoords(0+xr, 0-yd)
    g.draw(self.printer_bottom, 0+xr, 0-yd, 0, 1, 1, 0)

    for i,v in ipairs(self.printed_papers) do
        g.draw(v.paper, v.x+xr, v.y-yd, 0, 1, 1, v.paper:getWidth()/2)
    end

    g.draw(self.printer_img, 0+xr, 0-yd, 0, 1, 1)

    g.setColor(255, 255, 255, 255)

    g.setColor(255, 0, 0, 255)
    --g.print("ani time: " .. tostring(animation_time), 25, 75)
    --g.print("ys time: " .. tostring(ys), 25, 95)
    g.setColor(255, 255, 255, 255)

    self.lcd:draw()

    if (animation_time*0.30) < 3 then
        g.setColor(255, 255, 255, 0-pentagram)
        g.draw(self.pentagram, 0, 0, 0, 1, 1)

        g.setColor(255, 0, 0, 255)
        g.setFont(self.game.preloaded_fonts["04b03_24"])
        local string = "Defeat the printer!"
        local scale = (1+(math.sin(animation_time*2.5)*0.8))
        g.print(string, (screen_w/2), screen_h/2, 0, scale, scale, (#string/2)*12)


        scale = (1+(math.sin(animation_time*1.5)*0.1 ))
        string = "Click the paper throwing stars to bounce them back!"
        g.print(string, (screen_w/2), screen_h/2+80, 0, scale, scale, (#string/2)*13)
    end

    g.setColor(115, 34, 34, 0-pentagram)
    love.graphics.setLineWidth(3)
    g.rectangle("line", 30, screen_h-60, screen_w-60, 50)
    love.graphics.setLineWidth(1)
    g.setColor(217, 65, 65, 0-pentagram)
    g.rectangle("fill", 34, screen_h-56, hp_bar, 42)

    -- player life bar

    g.setColor(34, 115, 34, 0-pentagram)
    love.graphics.setLineWidth(3)
    g.rectangle("line", 30, screen_h-120, screen_w-60, 50)
    love.graphics.setLineWidth(1)
    g.setColor(65, 217, 65, 0-pentagram)
    g.rectangle("fill", 34, screen_h-116, player_hp_bar, 42)

    g.setColor(255, 255, 255, 255)

    for i,v in ipairs(stars) do
        if not v.disabled then
            if v.hit then
                g.draw(self.throw_star, v.x, v.y, -v.r, 1, 1, self.throw_star:getWidth()/2, self.throw_star:getHeight()/2)
                g.setColor(0, 255, 0, 64)
                g.draw(self.throw_star, v.x, v.y, -v.r, 1, 1, self.throw_star:getWidth()/2, self.throw_star:getHeight()/2)
            else
                g.draw(self.throw_star, v.x, v.y, v.r, 1, 1, self.throw_star:getWidth()/2, self.throw_star:getHeight()/2)
                if v.hover and not v.pressed then
                    g.setColor(255, 0, 0, 64)
                    g.draw(self.throw_star, v.x, v.y, v.r, 1, 1, self.throw_star:getWidth()/2, self.throw_star:getHeight()/2)
                end

                if v.pressed then
                    g.setColor(255, 0, 0, 128)
                    g.draw(self.throw_star, v.x, v.y, v.r, 1, 1, self.throw_star:getWidth()/2, self.throw_star:getHeight()/2)
                end
            end
            g.setColor(255, 255, 255, 255)
        end
    end


    local decimal = round(animation_time - math.floor(animation_time), 2)

    --[[
    if printer_hp == 0 then
        g.print("min_movement: ".. tostring(min_movement), 75, 75)
        g.print("max_movement: ".. tostring(max_movement), 75, 120)
        g.print("printer_dead_time: ".. tostring(animation_time - printer_dead_time), 75, 150)
    end
    --]]

    g.draw(self.cursor, mouse_x, mouse_y, 0, 1, 1, self.cursor:getWidth()/2, self.cursor:getHeight()/2)

    if math.random() < 0.3 and decimal > 0.42 and decimal < 0.44 and printer_hp > 0 then
        local redRandom = math.random()
        if redRandom > 0.7 then
            g.setColor(255, 0, 0, 64)
        else
            g.setColor(255, 255, 255, 64)
        end
        g.rectangle('fill', 0, 0, screen_w, screen_h)
    end
    g.setColor(255, 255, 255, 255)
end

function Exorcism:update(dt)
    animation_time = animation_time + dt
    mouse_x = love.mouse.getX()
    mouse_y = love.mouse.getY()
    love.mouse.setVisible(false)

    local decimal = round(animation_time - math.floor(animation_time), 2)
    for i,v in ipairs(stars) do
        if rd == 1 then
            v.r = (v.r - 0.05)
        else
            v.r = (v.r + 0.05)
        end

        if v.hit then
            v.x = v.x + v.xd*2.5
            v.y = v.y + v.yd*2.5
        else
            v.x = v.x - v.xd*v.speed
            v.y = v.y - v.yd*v.speed
        end

        if not v.hit then
            if inside_btn(mouse_x, mouse_y, v.x-(self.throw_star:getWidth()/2), v.y-(self.throw_star:getHeight()/2), self.throw_star:getWidth(), self.throw_star:getHeight()) then
                v.hover = true
            else
                v.hover = false
                v.pressed = false
            end

            if (v.x-(self.throw_star:getWidth()/2)) > (screen_w + self.throw_star:getWidth() + 15) or
                (v.y-(self.throw_star:getHeight()/2)) > (screen_h + self.throw_star:getHeight() + 15) or
                (v.x-(self.throw_star:getWidth()/2)) < (0 - self.throw_star:getWidth())-15 or
                (v.y-(self.throw_star:getHeight()/2)) < (0 - self.throw_star:getHeight())-15 then

                if not v.disabled then
                    v.disabled = true
                    self.hit_sound:play()
                    player_hp = player_hp - 0.075
                    if player_hp <= 0 then
                        player_hp = 0
                        if player_dead_time == 0 then
                            player_dead_time = animation_time
                            self.battle_music:stop()
                            self.game:activeGameOver()

                        end
                    end
                end
            end
        else
            if inside_btn(v.x+(self.throw_star:getWidth()/2), v.y+(self.throw_star:getHeight()/2), v.rx, v.ry, self.throw_star:getWidth(), self.throw_star:getHeight()) then
                if not v.disabled then
                    self.hit2_sound:play()
                    printer_hp = printer_hp - 0.075
                    if printer_hp <= 0 then
                        printer_hp = 0
                        if printer_dead_time == 0 then
                            printer_dead_time = animation_time
                        end
                    end
                end
                v.disabled = true
            end
        end
    end

    if printer_hp > 0 then
        if (animation_time*0.30) > 2.5 and decimal > 0.42 and decimal < 0.44 then
            dirx = math.random(-1, 1)
            while dirx == 0 do
                dirx  = math.random(-1, 1)
            end

            diry = math.random(-1, 1)
            while diry == 0 do
                diry = math.random(-1, 1)
            end

            startx = screen_w/2 + math.random(-150, 150)
            starty = screen_h-325

            speed = math.random(2, 3) + math.random()

            table.insert(stars, {
                x = startx,
                y = starty,
                xd = dirx ,
                yd = diry,
                rx = startx,
                ry = starty,
                speed = speed,
                r = 0,
                rd = math.random(1, 2),
                hit = false,
                hover = false,
                pressed = false,
                disabled = false
            })
        end
    else
        time_since_dead = animation_time - printer_dead_time
        local temp_movement_max = cap(10*(time_since_dead*0.20), 0, 10)
        max_movement = 10 - temp_movement_max

        local temp_movement_min = cap(5*(time_since_dead*0.20), 0, 5)
        min_movement = 5 - temp_movement_min

        local volume = 0.5 - cap(0.5*(time_since_dead*0.20), 0, 0.5)
        if volume == 0 then
            self.battle_music:stop()
        else
            self.battle_music:setVolume(volume)
        end

        if time_since_dead > 9 then
            self.exorcism_happend = true
            self.game:setCountdownEnabled(true)
            self:gotoState("WithPaper")
        end
    end

    self.lcd:update(dt)
end

return Exorcism
