local Firmware = Printer:addState("Firmware")

function Firmware:enteredState()
    animation_time = 0
    paper = nil

    box_selected = math.random(1,2)
    if box_selected == 1 then
        box_img = self.printer_firmware_box1
    else
        box_img = self.printer_firmware_box2
    end

    btn1_startx = 45
    btn1_hover = false
    btn1_pressed = false

    btn2_startx = 199
    btn2_hover = false
    btn2_pressed = false

    btns_starty = 103
    btns_width = 100
    btns_height = 53

    updating = false
    updating_time = 0
    update_ys = 0

    self.lcd:gotoState("Warning")
end

function Firmware:hasPaper()
    return true
end

function Firmware:hasInkt()
    return true
end

function Firmware:isHatchOpen()
    return false
end

function Firmware:setPaperHover(hover)
    paper_hover = hover
end

function Firmware:updateBtn()
    updating = true
    updating_time = animation_time
end

function Firmware:mousereleased(x, y, button, isTouch)
    if button == 1 then
        if btn1_pressed and btn1_hover then
            if box_selected == 1 then
                Firmware:updateBtn()
            else
                self:gotoState("WithPaper")
            end
            self.btn_sound:play()
        end

        if btn2_pressed and btn2_hover then
            if box_selected == 1 then
                self:gotoState("WithPaper")
            else
                Firmware:updateBtn()
            end
            self.btn_sound:play()
        end
    end
end

function Firmware:mousepressed(x, y, button, isTouch)
    if btn1_hover then
        btn1_pressed = true
    end

    if btn2_hover then
        btn2_pressed = true
    end
end

function Firmware:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(self.bg, 0, 0, 0, 1, 1)

    self.lcd:setPrinterCoords(0, 0)
    g.draw(self.printer_bottom, 0, 0, 0, 1, 1)

    for i,v in ipairs(self.printed_papers) do
        g.draw(v.paper, v.x, v.y, 0, 1, 1, v.paper:getWidth()/2)
    end

    g.draw(self.printer_img, 0, 0, 0, 1, 1)

    g.draw((updating and self.printer_firmware_update_box or box_img), 0, 0, 0, 1, 1)

    if updating then
        g.setColor(121, 139, 242, 255)
        g.rectangle('line', 45, 100, 250, 30)
        g.rectangle('fill', 48, 102, 243*update_ys, 25)
        g.setColor(255, 255, 255, 255)
    end

    g.draw(self.printer_firmware_lines, 30, 38, 0, 1, 1)

    if btn1_hover or btn2_hover then
        g.draw(self.cursor, mouse_x, mouse_y, 0, 1, 1, self.cursor:getWidth()/2, self.cursor:getHeight()/2)
    end

    g.setColor(255, 0, 0, 255)
    local bstr = "Firmware update! Quick select something!"
    g.print(bstr, (screen_w/2) - ((#bstr)*4.5)-100, screen_h-30)

    --g.rectangle('fill', 199, 103, 100, 53)

    --g.print("btn1_hover: " .. tostring(btn1_hover), 25, 75)
    --g.print("btn1_pressed: " .. tostring(btn1_pressed), 25, 100)

    --g.print("btn2_hover: " .. tostring(btn2_hover), 25, 125)
    --g.print("btn2_pressed: " .. tostring(btn2_pressed), 25, 150)
    --g.print("ys time: " .. tostring(ys), 25, 95)
    g.setColor(255, 255, 255, 255)
    self.lcd:draw()
end

function Firmware:update(dt)
    animation_time = animation_time + dt
    mouse_x = love.mouse.getX()
    mouse_y = love.mouse.getY()

    if updating then
        update_ys = cap(1*((animation_time-updating_time)*0.10), 0, 1)

        if update_ys == 1 then
            self.wifi_done_sound:play()
        end

        if (animation_time-updating_time)*0.10 > 1.05 then
            self:gotoState("WithPaper")
        end
    end

    -- btn 1 hover
    if inside_btn(mouse_x, mouse_y, btn1_startx, btns_starty, btns_width, btns_height) then
        btn1_hover = true
        love.mouse.setVisible(false)
    else
        btn1_hover = false
        btn1_pressed = false
        love.mouse.setVisible(true)

        -- btn 2 hover
        if inside_btn(mouse_x, mouse_y, btn2_startx, btns_starty, btns_width, btns_height) then
            btn2_hover = true
            love.mouse.setVisible(false)
        else
            btn2_hover = false
            btn2_pressed = false
            love.mouse.setVisible(true)
        end
    end

    self.lcd:update(dt)
end

return Firmware
