local NoInkt = Printer:addState("NoInkt")

function NoInkt:enteredState()
    animation_time = 0
    paper = nil

    has_old_inkt = true

    compartment_startx = 235
    compartment_starty = 318
    compartment_close_starty = compartment_starty - 104
    compartment_width = 330
    compartment_height = 102

    compartment_hover = false
    compartment_close_hover = false
    compartment_pressed = false
    compartment_close_pressed = false
    compartment_open = false

    old_inkt_startx = 237
    old_inkt_starty = 324
    old_inkt_width = 28
    old_inkt_height = 55
    old_inkt_hover = false
    old_inkt_pressed = false
    old_inkt_start_eject_time = 0

    old_inkt_eject = false
    old_inkt_eject_x = old_inkt_startx
    old_inkt_eject_y = old_inkt_starty

    new_inkt = false
    new_inkt_time = 0
    xr = 0
    animate_xr = 0

    mouse_x = love.mouse.getX()
    mouse_y = love.mouse.getY()
    self.lcd:gotoState("Warning")
end

function NoInkt:hasPaper()
    return true
end

function NoInkt:hasInkt()
    if new_inkt then
        return true
    end

    return false
end

function NoInkt:isHatchOpen()
    return compartment_open
end

function NoInkt:setPaperHover(hover)
    paper_hover = hover
end

function NoInkt:mousereleased(x, y, button, isTouch)
    if compartment_pressed and not new_inkt then
        compartment_open = true
        compartment_pressed = false
        compartment_close_pressed = false
        animate_xr = 2
        self.hit_sound:play()
    end

    if compartment_close_pressed then
        compartment_open = false
        compartment_close_pressed = false
        compartment_pressed = false
        animate_xr = 2
        self.hit_sound:play()

        if new_inkt then
            new_inkt_time = animation_time
        end
    end

    if old_inkt_pressed and not old_inkt_eject then
        old_inkt_pressed = false
        old_inkt_hover = false
        has_old_inkt = false
        old_inkt_eject = true
        animate_xr = 7
        old_inkt_start_eject_time = animation_time
        self.hit_sound:play()
    end
end

function NoInkt:mousepressed(x, y, button, isTouch)
    if compartment_hover then
        compartment_pressed = true
        compartment_close_pressed = false
    end

    if compartment_close_hover then
        compartment_close_pressed  = true
        compartment_pressed = false
    end

    if old_inkt_hover then
        old_inkt_pressed  = true
    end
end

function NoInkt:setInktHover(value)
    if old_inkt_eject and not new_inkt then
        if old_inkt_hover then
            new_inkt = value
            animate_xr = 7

            if not self.hit_sound:isPlaying() then
                self.hit_sound:play()
            end
            return true
        end
    end

    return false
end

function NoInkt:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(self.bg, 0, 0, 0, 1, 1)

    if animate_xr > 0 then
        animate_xr = animate_xr - 1
        xr = math.random(-2, 2)
    else
        xr = 0
    end

    self.lcd:setPrinterCoords(0+xr, 0)

    g.draw(self.printer_bottom, 0+xr, 0, 0, 1, 1)

    for i,v in ipairs(self.printed_papers) do
        g.draw(v.paper, v.x+xr, v.y, 0, 1, 1, v.paper:getWidth()/2)
    end

    g.draw(self.printer_img, 0+xr, 0, 0, 1, 1)

    if compartment_open then
        g.draw(((has_old_inkt or new_inkt) and self.hatch_open or self.hatch_open_no_inkt), compartment_startx+xr, compartment_starty-104, 0, 1, 1)
    end

    if compartment_hover or compartment_close_hover or (old_inkt_hover and has_old_inkt and not old_inkt_eject) then
        g.draw(self.cursor, mouse_x, mouse_y, 0, 1, 1, self.cursor:getWidth()/2, self.cursor:getHeight()/2)
    end

    if old_inkt_eject and old_inkt_eject_y < (screen_h+self.broken_inkt:getHeight()) then
        local ys = cap((-1.2)+((animation_time-old_inkt_start_eject_time)*2.5), -1.2, 3.5)

        old_inkt_eject_y = old_inkt_eject_y + (4*ys)
        old_inkt_eject_x = old_inkt_eject_x + 2

        g.draw(self.broken_inkt, old_inkt_eject_x, old_inkt_eject_y, 0, 1, 1, self.broken_inkt:getWidth()/2, self.broken_inkt:getHeight()/2)
    end

    g.setColor(255, 0, 0, 255)

    local bstr = "No inkt! Quick switch the inkt!"
    g.print(bstr, (screen_w/2) - ((#bstr)*4.5)-50, screen_h-30)
    --g.rectangle('fill', 237, 324, 28, 55)
    --g.print("ani time: " .. tostring(animation_time), 25, 75)

    g.setColor(255, 255, 255, 255)
    self.lcd:draw()
end

function NoInkt:update(dt)
    animation_time = animation_time + dt

    mouse_x = love.mouse.getX()
    mouse_y = love.mouse.getY()

    if not compartment_open then
        compartment_close_hover = false
        if new_inkt then
            local time = animation_time - new_inkt_time
            if time > 1.2 then
                self:gotoState("WithPaper")
            end
        end

        if mouse_x >= compartment_startx and mouse_x <= compartment_startx + compartment_width then
            if mouse_y >= compartment_starty and mouse_y <= compartment_starty + compartment_height then
                compartment_hover = true
                love.mouse.setVisible(false)
            else
                compartment_hover = false
                compartment_pressed = false
                love.mouse.setVisible(true)
            end
        else
            love.mouse.setVisible(true)
            compartment_hover = false
            compartment_pressed = false
        end
    else
        compartment_hover = false
        if mouse_x >= compartment_startx and mouse_x <= compartment_startx + compartment_width then
            if mouse_y >= compartment_close_starty and mouse_y <= compartment_close_starty + compartment_height then
                compartment_close_hover = true
                love.mouse.setVisible(false)
            else
                compartment_close_hover = false
                compartment_close_pressed = false
                love.mouse.setVisible(true)
            end
        else
            love.mouse.setVisible(true)
            compartment_close_hover = false
            compartment_close_pressed = false
        end

        if not compartment_close_hover and not compartment_close_pressed then
            if mouse_x >= old_inkt_startx and mouse_x <= old_inkt_startx + old_inkt_width then
                if mouse_y >= old_inkt_starty and mouse_y <= old_inkt_starty + old_inkt_height then
                    old_inkt_hover = true
                    if not old_inkt_eject then
                        love.mouse.setVisible(false)
                    end
                else
                    old_inkt_hover = false
                    if not old_inkt_eject then
                        love.mouse.setVisible(true)
                    end
                end
            else
                if not old_inkt_eject then
                    love.mouse.setVisible(true)
                end
                old_inkt_hover = false
            end
        end
    end

    self.lcd:update(dt)
end

return NoInkt
