local NoPowerCable = Printer:addState("NoPowerCable")

function NoPowerCable:enteredState()
    animation_time = 0
    paper = nil

    power_plug_target_x = 0
    power_plug_target_y = 0
    power_plug_x = 0
    power_plug_y = 0

    a_pressed = false
    d_pressed = false
    w_pressed = false
    s_pressed = false
    last_x_dir = 0
    last_y_dir = 0

    socket_start_x = 45
    socket_start_y = 115
    socket_width = 75
    socket_height = 45

    percent_done = 0.01
    cord_connected = false
    cord_connected_time = 0

    self.power_drain:setVolume(0.25)
    self.power_drain:play()

    self.lcd:gotoState("NoPower")
end

function NoPowerCable:hasPaper()
    return true
end

function NoPowerCable:hasInkt()
    return true
end

function NoPowerCable:setPaperHover(hover)
    paper_hover = hover
end

function NoPowerCable:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(self.bg, 0, 0, 0, 1, 1)
    xr = (math.random()*2.5)

    g.draw(self.printer_bottom, 0, 0, 0, 1, 1)

    for i,v in ipairs(self.printed_papers) do
        g.draw(v.paper, v.x, v.y, 0, 1, 1, v.paper:getWidth()/2)
    end

    g.draw(self.printer_img, 0, 0, 0, 1, 1)

    if not cord_connected then
        g.draw(self.printer_backside_box, 0, 0, 0, 1, 1)

        g.draw(self.power_plug_head, power_plug_x, power_plug_y, 0, 1, 1, self.power_plug_head:getWidth()/2, self.power_plug_head:getHeight()/2)
        x = 0
        dir = 1
        for i = power_plug_y, 186 do
            --g.print("ys time: " .. tostring(ys), 25, 95)
            g.setColor(255, 255, 255, 255)
            g.draw(self.power_plug_part, power_plug_x-x, i+(self.power_plug_head:getHeight()/2)+(self.power_plug_part:getHeight()/2), 0, 1, 1, self.power_plug_part:getWidth()/2, self.power_plug_part:getHeight()/2)

            if x == 0 and dir == 1 then
                x = 1
            elseif x == 0 and dir == 0 then
                x = -1
            elseif x == 1 and dir == 1 then
                x = 0
                dir = 0
            elseif x == -1 and dir == 0 then
                x = 0
                dir = 1
            end
        end

        g.setColor(161, 49, 49, 255)
        g.rectangle('fill', 352, 15, 21, 183*percent_done)
    else
        g.draw(self.printer_backside_box_plugged, 0, 0, 0, 1, 1)
    end

    g.setColor(255,0,0,255)
    local bstr = "Power cord unplugged! Use W A S D to plug it back in!"
    g.print(bstr, (screen_w/2) - ((#bstr)*4.5)-100, screen_h-30)
    --g.print("percent: " .. tostring(percent_done), 75, 75)
    g.setColor(255, 255, 255, 255)
    self.lcd:draw()
end

function NoPowerCable:mousemoved(x, y, dx, dy)
    --cur_mouse_x = x
    --cur_mouse_y = y
end

function NoPowerCable:keypressed(key, scancode, isrepeat)
    if key == 'a' then
        a_pressed = true
        power_plug_target_x = power_plug_target_x - (5 + math.random(-5,5))
        last_x_dir = -1
        if power_plug_target_x < 25 then
            power_plug_target_x = 25
        end
    end

    if key == 'd' then
        d_pressed = true
        power_plug_target_x = power_plug_target_x + (5 + math.random(-5,5))
        last_x_dir = 1
        if power_plug_target_x > 310 then
            power_plug_target_x = 310
        end
    end

    if key == 'w' then
        w_pressed = true
        power_plug_target_y = power_plug_target_y - (5 + math.random(-5,5))
        last_y_dir = -1
        if power_plug_target_y < 25 then
            power_plug_target_y = 25
        end
    end

    if key == 's' then
        s_pressed = true
        power_plug_target_y = power_plug_target_y + (5 + math.random(-5,5))
        last_y_dir = 1
        if power_plug_target_y > 187 then
            power_plug_target_y = 187
        end
    end
end

function NoPowerCable:keyreleased(key, scancode)
    if key == 'a' then
        a_pressed = false
    end

    if key == 'd' then
        d_pressed = false
    end

    if key == 'w' then
        w_pressed = false
    end

    if key == 's' then
        s_pressed = false
    end
end

function NoPowerCable:update(dt)
    animation_time = animation_time + dt

    if cord_connected then
        time_connected = animation_time - cord_connected_time
        if time_connected > 2 then
            self:gotoState("WithPaper")
        end
    end

    if a_pressed == true then
        power_plug_target_x = power_plug_target_x - (5 + math.random(-5,5))
        if power_plug_target_x < 25 then
            power_plug_target_x = 25
        end
    end

    if d_pressed == true then
        power_plug_target_x = power_plug_target_x + (5 + math.random(-5,5))
        if power_plug_target_x > 310 then
            power_plug_target_x = 310
        end
    end

    if w_pressed == true then
        power_plug_target_y = power_plug_target_y - (5 + math.random(-5,5))
        if power_plug_target_y < 25 then
            power_plug_target_y = 25
        end
    end

    if s_pressed == true then
        power_plug_target_y = power_plug_target_y + (5 + math.random(-5,5))
        if power_plug_target_y > 187 then
            power_plug_target_y = 187
        end
    end

    if last_x_dir == -1 then
        power_plug_target_x = power_plug_target_x - (5 + math.random(-5,5))
        if power_plug_target_x < 25 then
            power_plug_target_x = 25
        end
    elseif last_x_dir == 1 then
        power_plug_target_x = power_plug_target_x + (5 + math.random(-5,5))
        if power_plug_target_x > 310 then
            power_plug_target_x = 310
        end
    end

    if last_y_dir == -1 then
        power_plug_target_y = power_plug_target_y - (5 + math.random(-5,5))
        if power_plug_target_y < 25 then
            power_plug_target_y = 25
        end
    elseif last_y_dir == 1 then
        power_plug_target_y = power_plug_target_y + (5 + math.random(-5,5))
        if power_plug_target_y > 187 then
            power_plug_target_y = 187
        end
    end

    power_plug_x = lerp(power_plug_x, power_plug_target_x, dt*1.5)
    power_plug_y = lerp(power_plug_y, power_plug_target_y, dt*1.5)

    if power_plug_x >= socket_start_x and power_plug_x <= socket_start_x + socket_width then
        if power_plug_y >= socket_start_y and power_plug_y <= socket_start_y + socket_height then
            percent_done = lerp(percent_done, percent_done+0.15, dt*1.5)
            if percent_done > 1 then
                percent_done = 1
                cord_connected = true
                cord_connected_time = animation_time
            end
        else
            percent_done = lerp(percent_done, percent_done-0.10, dt)
            if percent_done < 0 then
                percent_done = 0
            end
        end
    else
        percent_done = lerp(percent_done, percent_done-0.10, dt)
        if percent_done < 0 then
            percent_done = 0
        end
    end

    self.lcd:update(dt)
end

return NoPowerCable
