local PaperStuck = Printer:addState("PaperStuck")

function randomCheck(tab, val)
    for index, value in ipairs (tab) do
        if value['val'] == val then
            return true
        end
    end

    return false
end

function PaperStuck:enteredState()
    animation_time = 0
    paper = nil

    has_old_inkt = true

    compartment_startx = 235
    compartment_starty = 318
    compartment_close_starty = compartment_starty - 104
    compartment_width = 330
    compartment_height = 102

    compartment_hover = false
    compartment_close_hover = false
    compartment_pressed = false
    compartment_close_pressed = false
    compartment_open = false

    stuck_paper_hover = false
    stuck_paper_pressed = false
    stuck_paper_start_touchx = 0
    stuck_paper_start_touchy = 0
    stuck_paper_radius = math.random(75, 175)
    stuck_break_points = {}
    for i=1,4 do
        randomness = math.random(-2, 0)
        while randomCheck(stuck_break_points, randomness) do
            randomness = math.random(-2, 0)
        end
        table.insert(stuck_break_points, {
            done = false,
            val = round(randomness + math.random(), 3)
        })
    end

    paper_pieces = {}
    for i=1,math.random(9,28) do
        table.insert(paper_pieces, {
            x = compartment_startx+6+math.random(0,self.stuck_paper[1]:getWidth()),
            y = compartment_starty+48+math.random(0,self.stuck_paper[1]:getHeight()),
            yd = round(math.random(),1),
            r = math.random(),
            rd = round(math.random(),1)
        })
    end

    old_mouse_x = love.mouse.getX()
    old_mouse_y = love.mouse.getY()
    real_mouse_x = love.mouse.getX()
    real_mouse_y = love.mouse.getY()

    xr = 0
    animate_xr = 0
    animate_paper_exposion = false
    paper_removed = false

    test_atan2 = 0
    self.lcd:gotoState("Warning")
end

function PaperStuck:hasPaper()
    return true
end

function PaperStuck:hasInkt()
    return true
end

function PaperStuck:isHatchOpen()
    return compartment_open
end

function PaperStuck:setPaperHover(hover)
    paper_hover = hover
end

function PaperStuck:mousereleased(x, y, button, isTouch)
    if compartment_pressed then
        compartment_open = true
        compartment_pressed = false
        compartment_close_pressed = false
    end

    if compartment_close_pressed then
        compartment_open = false
        compartment_close_pressed = false
        compartment_pressed = false

        if paper_removed then
            self:gotoState("WithPaper")
        end
    end

    if stuck_paper_pressed then
        stuck_paper_pressed = false
    end
end

function PaperStuck:mousepressed(x, y, button, isTouch)
    if compartment_hover then
        compartment_pressed = true
        compartment_close_pressed = false
    end

    if compartment_close_hover then
        compartment_close_pressed  = true
        compartment_pressed = false
    end

    if stuck_paper_hover then
        if not stuck_paper_pressed then
            stuck_paper_start_touchx = love.mouse.getX()
            stuck_paper_start_touchy = love.mouse.getY()
        end
        compartment_close_pressed  = false
        compartment_pressed = false
        stuck_paper_pressed = true
    end
end

function PaperStuck:setInktHover(value)
    return false
end

function PaperStuck:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(self.bg, 0, 0, 0, 1, 1)

    if animate_xr > 0 then
        animate_xr = animate_xr - 1
        xr = math.random(-2, 2)
    else
        xr = 0
    end

    self.lcd:setPrinterCoords(0+xr, 0)
    g.draw(self.printer_bottom, 0+xr, 0, 0, 1, 1)

    for i,v in ipairs(self.printed_papers) do
        g.draw(v.paper, v.x+xr, v.y, 0, 1, 1, v.paper:getWidth()/2)
    end

    g.draw(self.printer_img, 0+xr, 0, 0, 1, 1)

    count_done = 1
    for i=1,#stuck_break_points do
        if stuck_break_points[i]['done'] then
            count_done = count_done + 1
        end
    end

    if compartment_open then
        g.draw(self.hatch_open, compartment_startx+xr, compartment_starty-104, 0, 1, 1)

        if count_done < 5 then
            g.draw(self.stuck_paper[count_done], compartment_startx+6+xr, compartment_starty+48, 0 , 1, 1)

            if stuck_paper_pressed then
                g.setLineWidth(2)
                g.setColor(255, 0, 0, 255)
                g.line(stuck_paper_start_touchx, stuck_paper_start_touchy, real_mouse_x, real_mouse_y)
                g.setColor(255, 255, 255, 255)
                g.setLineWidth(1)
            end
        end

        if animate_paper_exposion then
            local ys = cap((-1.2)+((animation_time-animate_paper_exposion_start_time)*2.5), -1.2, 3.5)

            for i=1, #paper_pieces do
                if paper_pieces[i]['y'] + self.paper_piece:getHeight() < screen_h + 5 then
                    paper_pieces[i]['y'] = paper_pieces[i]['y'] + (4*ys)
                    paper_pieces[i]['x'] = paper_pieces[i]['x'] + (2*(paper_pieces[i]['yd'] < 0.5 and -1 or 1))
                    paper_pieces[i]['r'] = (paper_pieces[i]['r'] + (1+math.sin(animation_time))*0.2)
                    g.draw(self.paper_piece, paper_pieces[i]['x'], paper_pieces[i]['y'], paper_pieces[i]['r']*(paper_pieces[i]['rd'] < 0.5 and -1 or 1), 1, 1, self.paper_piece:getWidth()/2, self.paper_piece:getHeight()/2)
                end
            end
        end
    end

    if compartment_hover or compartment_close_hover or (stuck_paper_hover and count_done < 5) or (stuck_paper_pressed and count_done < 5) then
        g.draw(self.cursor, mouse_x, mouse_y, 0, 1, 1, self.cursor:getWidth()/2, self.cursor:getHeight()/2)
    end

    g.setColor(255, 0, 0, 255)
    local bstr = "Paper stuck! Quick pull it out!"
    g.print(bstr, (screen_w/2) - ((#bstr)*4.5)-50, screen_h-30)
    g.setColor(255, 255, 255, 255)

    g.setColor(255, 0, 0, 128)


--[[
    for i=1,#stuck_break_points do
        if stuck_break_points[i]['done'] then
            g.setColor(0, 255, 0, 128)
        end
        g.print("Point ".. tostring(i) .. ": " .. tostring(stuck_break_points[i]['val']), 25, 120+(25*i))
        g.setColor(255, 0, 0, 128)
    end
--]]

    --g.print("dist: " .. tostring(dist), 25, 100)

    g.setColor(255, 255, 255, 255)
    self.lcd:draw()
end

function PaperStuck:mousemoved(x, y, dx, dy)
    mouse_x_cur = x
    mouse_y_cur = y
end

function PaperStuck:update(dt)
    animation_time = animation_time + dt

    old_mouse_x = real_mouse_x
    old_mouse_y = real_mouse_y

    count_done = 1
    for i=1,#stuck_break_points do
        if stuck_break_points[i]['done'] then
            count_done = count_done + 1
        end
    end

    if stuck_paper_pressed then
        mouse_x_cur = love.mouse.getX()
        mouse_y_cur = love.mouse.getY()

        if not pointInCircleCords(mouse_x_cur, mouse_y_cur, stuck_paper_start_touchx, stuck_paper_start_touchy, stuck_paper_radius) then
            test_atan2 = round(math.atan2(mouse_y_cur - stuck_paper_start_touchy, mouse_x_cur - stuck_paper_start_touchx), 3)

            real_mouse_x = stuck_paper_start_touchx + stuck_paper_radius * math.cos(test_atan2)
            real_mouse_y = stuck_paper_start_touchy + stuck_paper_radius * math.sin(test_atan2)

            for i=1,#stuck_break_points do
                if not stuck_break_points[i]['done'] then
                    if test_atan2 >= (stuck_break_points[i]['val'] - 0.05) and test_atan2 <= (stuck_break_points[i]['val'] + 0.05) then
                        stuck_break_points[i]['done'] = true
                        animate_xr = 5
                        self.paper_break_sound:play()
                    end
                end
            end

            if count_done == 5 then
                animate_paper_exposion = true
                paper_removed = true
                animate_paper_exposion_start_time = animation_time
            end
        else
            real_mouse_x = mouse_x_cur
            real_mouse_y = mouse_y_cur
        end

        --mouse_x = stuck_paper_start_touchx
        --mouse_y = stuck_paper_start_touchy
    else
        real_mouse_x = love.mouse.getX()
        real_mouse_y = love.mouse.getY()
    end

    if not compartment_open then
        compartment_close_hover = false
        if mouse_x >= compartment_startx and mouse_x <= compartment_startx + compartment_width then
            if mouse_y >= compartment_starty and mouse_y <= compartment_starty + compartment_height then
                compartment_hover = true
                love.mouse.setVisible(false)
            else
                compartment_hover = false
                compartment_pressed = false
                love.mouse.setVisible(true)
            end
        else
            love.mouse.setVisible(true)
            compartment_hover = false
            compartment_pressed = false
        end
    else
        compartment_hover = false
        if mouse_x >= compartment_startx and mouse_x <= compartment_startx + compartment_width then
            if mouse_y >= compartment_close_starty and mouse_y <= compartment_close_starty + compartment_height then
                compartment_close_hover = true
                love.mouse.setVisible(false)
            else
                compartment_close_hover = false
                compartment_close_pressed = false
                love.mouse.setVisible(true)
            end
        else
            love.mouse.setVisible(true)
            compartment_close_hover = false
            compartment_close_pressed = false
        end

        if count_done < 5 then
            if mouse_x >= (compartment_startx+6) and mouse_x <= (compartment_startx+6) + self.stuck_paper[1]:getWidth() then
                if mouse_y >= (compartment_starty+48) and mouse_y <= (compartment_starty+48) + self.stuck_paper[1]:getHeight() then
                    stuck_paper_hover = true
                    love.mouse.setVisible(false)
                else
                    stuck_paper_hover = false
                    --stuck_paper_pressed = false
                    if not stuck_paper_pressed and not compartment_close_hover then
                        love.mouse.setVisible(true)
                    end
                end
            else
                stuck_paper_hover = false
                --stuck_paper_pressed = false
                if not stuck_paper_pressed and not compartment_close_hover then
                    love.mouse.setVisible(true)
                end
            end
        end
    end

    if stuck_paper_pressed then
        love.mouse.setVisible(false)
    end

    self.lcd:update(dt)
end

return PaperStuck
