local NoWifi = Printer:addState("NoWifi")

function NoWifi:enteredState()
    animation_time = 0
    paper = nil

    box_img = self.printer_lcd_box

    updating = false
    updating_time = 0
    update_ys = 0

    max_len = 17
    possible_passwords = {
        "123456",
        "password",
        "12345678",
        "querty",
        "12345",
        "123456789",
        "football",
        "1234",
        "baseball",
        "welcome",
        "abc123",
        "dragon",
        "9843pass",
        "habbo123",
        "master",
        "monkey",
        "letmein",
        "hackz",
        "1337",
        "nospamplz",
        "420yolo"
    }


    correct_pass = string.lower(possible_passwords[math.random(#possible_passwords)])
    correct_pass_time = 0
    password = ""

    chars = "1234567890abcdefghijklmnopqrstuvwxyz"
    char_btns = {}
    for i = 1, #chars do
        local c = chars:sub(i,i)
        char_btns[c] = {
            hover = false,
            pressed = false
        }
    end
    char_btns['back'] = {
        hover = false,
        pressed = false
    }

    mouse_x = love.mouse.getX()
    mouse_y = love.mouse.getY()

    self.lcd:gotoState("Warning")
end

function NoWifi:hasPaper()
    return true
end

function NoWifi:hasInkt()
    return true
end

function NoWifi:isHatchOpen()
    return false
end

function NoWifi:setPaperHover(hover)
    paper_hover = hover
end

function NoWifi:mousereleased(x, y, button, isTouch)
    if button == 1 then
        if password ~= correct_pass then
            for i = 1, #chars do
                local c = chars:sub(i,i)
                if math.random() < 0.3 then
                    if math.random() < 0.5 then
                        x = 0
                        if i > 13 then
                            if i > 26 then
                                local xd = i - 25
                                x = 13 + xd
                            else
                                x = i - 13
                            end
                        else
                            x = i + 13
                        end
                    else
                        x = (math.random() < 0.5 and i-1 or i+1)
                    end
                else
                    x = i
                end

                local rchar = chars:sub(x, x)

                if char_btns[c]['pressed'] then
                    if (#password+1) < max_len then
                        password = password .. rchar

                        if password ~= correct_pass then
                            self.btn_sound:play()
                        else
                            self.wifi_done_sound:play()
                            correct_pass_time = animation_time

                        end
                    end
                end
            end

            if char_btns['back']['pressed'] then
                password = password:sub(1, #password-1)
                self.btn_sound:play()
            end
        end
    end
end

function NoWifi:mousepressed(x, y, button, isTouch)
    if button == 1 then
        for i = 1, #chars do
            local c = chars:sub(i,i)
            if char_btns[c]['hover'] then
                char_btns[c]['pressed'] = true
            end
        end

        if char_btns['back']['hover'] then
            char_btns['back']['pressed'] = true
        end
    end
end

function NoWifi:draw()
    g.setColor(255, 255, 255, 255)
    g.draw(self.bg, 0, 0, 0, 1, 1)

    self.lcd:setPrinterCoords(0, 0)
    g.draw(self.printer_bottom, 0, 0, 0, 1, 1)

    for i,v in ipairs(self.printed_papers) do
        g.draw(v.paper, v.x, v.y, 0, 1, 1, v.paper:getWidth()/2)
    end

    g.draw(self.printer_img, 0, 0, 0, 1, 1)

    g.draw(self.printer_lcd_box, 0, 0, 0, 1, 1)

    g.setColor(121, 139, 242, 255)

    g.setFont(self.game.preloaded_fonts["04b03_24"])

    g.print("No wifi!", 125, 40)
    g.setFont(self.game.preloaded_fonts["04b03_16"])
    g.print("put in you wifi password:", 60, 65)

    g.setFont(self.game.preloaded_fonts["04b03_24"])
    g.print(password, 50, 85)

    x = 45
    y = 120

    g.setFont(self.game.preloaded_fonts["04b03_14"])
    for i = 1, #chars do
        local c = chars:sub(i,i)

        if char_btns[c]['hover'] then
            g.setColor(121, 139, 242, 255)
            g.rectangle('line', x, y, 15, 15)
            g.setColor(121, 139, 242, 255)
            g.print(c, x+2, y+2)
        else
            g.setColor(121, 139, 242, 255)
            g.rectangle('fill', x, y, 15, 15)
            g.setColor(84, 102, 204, 255)
            g.print(c, x+4, y+1)
        end

        x = x + 20

        if i == 13 or i == 26 or i == 39 then
            x = 45
            if i == 26 then
                x = 65
            end
            y = y + 20
        end
    end


    if char_btns['back']['hover'] then
        g.setColor(121, 139, 242, 255)
        g.rectangle('line', x, y, 35, 15)
        g.setColor(121, 139, 242, 255)
        g.print('back', x+2, y+2)
    else
        g.setColor(121, 139, 242, 255)
        g.rectangle('fill', x, y, 35, 15)
        g.setColor(84, 102, 204, 255)
        g.print('back', x+2, y+2)
    end

    if updating then
        g.rectangle('line', 45, 100, 250, 30)
        g.rectangle('fill', 48, 102, 243*update_ys, 25)
        g.setColor(255, 255, 255, 255)
    end

    g.setColor(255, 255, 255, 255)
    g.draw(self.printer_firmware_lines, 30, 38, 0, 1, 1)

    g.setColor(255, 0, 0, 255)
    g.setFont(self.game.preloaded_fonts["04b03_24"])
    local string = "Your wifi password is: " .. correct_pass
    local scale = (1+(math.sin(animation_time*0.7)*0.1))

    g.print(string, (screen_w/2), screen_h-70, 0, scale, scale, (#string/2)*12)

    --if btn1_hover or btn2_hover then
        g.setColor(255, 255, 255, 255)
        love.mouse.setVisible(false)
        g.draw(self.cursor, mouse_x, mouse_y, 0, 1, 1, (self.cursor:getWidth()/2)-5, (self.cursor:getHeight()/2)-15)
        g.setColor(255, 255, 255, 255)
    --end

    g.setColor(255, 0, 0, 128)

    g.setColor(255, 0, 0, 255)
    local bstr = "No wifi! Quick put in your password!"
    g.print(bstr, (screen_w/2) - ((#bstr)*4.5)-50, screen_h-30)
    g.setColor(255, 255, 255, 255)

    --g.rectangle('fill', 199, 103, 100, 53)

    --g.print("btn1_hover: " .. tostring(btn1_hover), 25, 75)
    --g.print("btn1_pressed: " .. tostring(btn1_pressed), 25, 100)

    --g.print("btn2_hover: " .. tostring(btn2_hover), 25, 125)
    --g.print("btn2_pressed: " .. tostring(btn2_pressed), 25, 150)
    --g.print("mouse x: " .. tostring(mouse_x), 25, 95)
    --g.print("mouse y: " .. tostring(mouse_y), 25, 125)
    g.setColor(255, 255, 255, 255)
    self.lcd:draw()
end

function NoWifi:update(dt)
    animation_time = animation_time + dt
    mouse_x = love.mouse.getX()
    mouse_y = love.mouse.getY()

    x = 45
    y = 120

    if password == correct_pass then
        time = animation_time - correct_pass_time
        if time > 1 then
            self:gotoState("WithPaper")
        end
    end

    for i = 1, #chars do
        local c = chars:sub(i,i)
        if inside_btn(mouse_x, mouse_y, x, y, 15, 15) then
            char_btns[c]['hover'] = true
        else
            char_btns[c]['hover'] = false
            char_btns[c]['pressed'] = false
        end

        x = x + 20

        if i == 13 or i == 26 or i == 39 then
            x = 45
            if i == 26 then
                x = 65
            end
            y = y + 20
        end
    end

    if inside_btn(mouse_x, mouse_y, x, y, 35, 15) then
        char_btns['back']['hover'] = true
    else
        char_btns['back']['hover'] = false
        char_btns['back']['pressed'] = false
    end

    self.lcd:update(dt)
end

return NoWifi
